=== modified file 'configure.ac'
--- configure.ac	2009-03-04 02:32:20 +0000
+++ configure.ac	2009-03-04 02:48:12 +0000
@@ -459,6 +459,25 @@
       AC_MSG_ERROR([libz is required for Drizzle]))
 
 #--------------------------------------------------------------------
+# Check for TBB
+#--------------------------------------------------------------------
+
+AC_LANG_PUSH(C++)
+AC_LIB_HAVE_LINKFLAGS(tbb,,
+[#include <tbb/atomic.h>
+ #include <stdint.h>
+],
+[
+  tbb::atomic<uint64_t> x;
+  tbb::atomic<uint8_t> y;
+  x=0;
+  y=0;
+  x++;
+  y++;
+])
+AC_LANG_POP()
+
+#--------------------------------------------------------------------
 # Check for libreadline or compatible (libedit on Mac OS X)
 #--------------------------------------------------------------------
 
@@ -795,8 +814,7 @@
                   [Assume single-CPU mode, no concurrency]) ;;
   "rwlocks") AC_DEFINE([MY_ATOMIC_MODE_RWLOCKS], [1],
                   [Use pthread rwlocks for atomic ops]) ;;
-  "smp") ;;
-  "")
+  "smp") 
     AC_CACHE_CHECK(
       [whether the compiler provides atomic builtins],
       [ac_cv_gcc_atomic_builtins],

=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2009-03-04 02:28:24 +0000
+++ drizzled/Makefile.am	2009-03-04 02:48:12 +0000
@@ -30,6 +30,10 @@
 nobase_dist_drizzledinclude_HEADERS = \
 		alter_column.h \
 		alter_drop.h \
+		atomic/gcc_traits.h \
+		atomic/pthread_traits.h \
+		atomic/sun_studio.h \
+		atomics.h \
 		authentication.h \
 		base.h \
 		cached_item.h \
@@ -355,7 +359,7 @@
 		 $(top_builddir)/drizzled/serialize/libserialize.la \
 		 @mysql_plugin_libs@ \
 		 $(LDADD) $(LIBUUID) ${top_builddir}/gnulib/libgnu.la \
-		 $(LIBDL_LIBS) $(LIBPROTOBUF) $(LIBPCRE) \
+		 $(LIBDL_LIBS) $(LIBPROTOBUF) $(LIBPCRE) $(LIBTBB) \
 		 $(DRIZZLED_PLUGIN_DEP_LIBS)
 drizzled_DEPENDENCIES = @mysql_plugin_libs@ ${noinst_LTLIBRARIES}
 

=== added directory 'drizzled/atomic'
=== added file 'drizzled/atomic/gcc_traits.h'
--- drizzled/atomic/gcc_traits.h	1970-01-01 00:00:00 +0000
+++ drizzled/atomic/gcc_traits.h	2009-03-04 02:48:12 +0000
@@ -0,0 +1,85 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_ATOMIC_GCC_TRAITS_H
+#define DRIZZLED_ATOMIC_GCC_TRAITS_H
+
+namespace tbb {
+namespace internal {
+
+template<typename T, typename D>
+class gcc_traits
+{
+
+public:
+  typedef T value_type;
+
+  gcc_traits() {}
+
+  /* YES. I know these are semantically backwards...
+   * so... TODO: Ensure we're doing the "right" thing here
+   */
+  inline value_type fetch_and_add(volatile value_type *value, D addend )
+  {
+    return __sync_add_and_fetch(value, addend);
+  }
+
+  inline value_type fetch_and_increment(volatile value_type *value)
+  {
+    return __sync_add_and_fetch(value, 1);
+  }
+
+  inline value_type fetch_and_decrement(volatile value_type *value)
+  {
+    return __sync_sub_and_fetch(value, 1);
+  }
+
+  inline value_type fetch_and_store(volatile value_type *value,
+                                    value_type new_value)
+  {
+    /* TODO: Is this the right one? */
+    return __sync_lock_test_and_set(value, new_value);
+  }
+
+  inline value_type compare_and_swap(volatile value_type *value,
+                                     value_type new_value,
+                                     value_type comparand )
+  {
+    return __sync_val_compare_and_swap(value, comparand, new_value);
+  }
+
+  inline value_type fetch(const volatile value_type *value) const volatile
+  {
+    return __sync_add_and_fetch(const_cast<value_type *>(value), 0);
+  }
+
+  inline value_type store_with_release(volatile value_type *value,
+                                       value_type new_value)
+  {
+    *value= new_value;
+    return *value;
+  }
+
+}; /* gcc_traits */
+
+
+} /* namespace internal */
+} /* namespace tbb */
+
+#endif /* DRIZZLED_ATOMIC_GCC_TRAITS_H */

=== added file 'drizzled/atomic/pthread_traits.h'
--- drizzled/atomic/pthread_traits.h	1970-01-01 00:00:00 +0000
+++ drizzled/atomic/pthread_traits.h	2009-03-04 02:48:12 +0000
@@ -0,0 +1,139 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *  Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_ATOMIC_PTHREAD_TRAITS_H
+#define DRIZZLED_ATOMIC_PTHREAD_TRAITS_H
+
+namespace tbb {
+
+namespace internal {
+
+class mutex_wrapper
+{
+private:
+  pthread_mutex_t the_mutex;
+  bool locked;
+public:
+  mutex_wrapper(void): locked(false)
+  {
+    locked= false;
+    (void) pthread_mutex_init(&the_mutex,  NULL);
+  }
+  ~mutex_wrapper(void)
+  {
+    if (locked)
+      unlock();
+    pthread_mutex_destroy(&the_mutex);
+  }
+  void lock(void)
+  {
+    pthread_mutex_lock(&the_mutex);
+    locked=true;
+  }
+  void unlock(void)
+  {
+    pthread_mutex_unlock(&the_mutex);
+    locked=false;
+  }
+};
+
+template<typename T, typename D>
+class pthread_traits
+{
+private:
+  mutex_wrapper my_lock;
+
+public:
+
+  typedef T value_type;
+
+  pthread_traits() {}
+
+  inline value_type fetch_and_add(volatile value_type *value, D addend )
+  {
+    my_lock.lock();
+    *value += addend;
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
+  inline value_type fetch_and_increment(volatile value_type *value)
+  {
+    my_lock.lock();
+    *value++;
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
+  inline value_type fetch_and_decrement(volatile value_type *value)
+  {
+    my_lock.lock();
+    *value--;
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
+  inline value_type fetch_and_store(volatile value_type *value,
+                                    value_type new_value )
+  {
+    my_lock.lock();
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
+  inline value_type compare_and_swap(volatile value_type *value,
+                                     value_type new_value,
+                                     value_type comparand )
+  {
+    my_lock.lock();
+    if (*value == comparand)
+      *value= new_value;
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
+  inline value_type fetch(const volatile value_type *value) const volatile
+  {
+    return *value;
+  }
+
+  inline value_type store_with_release(volatile value_type *value,
+                                       value_type new_value)
+  {
+    my_lock.lock();
+    *value= new_value;
+    value_type ret= *value;
+    my_lock.unlock();
+    return ret;
+  }
+
+}; /* pthread_traits */
+
+
+} /* namespace internal */
+} /* namespace tbb */
+
+
+#endif /* DRIZZLED_ATOMIC_PTHREAD_TRAITS_H */

=== added file 'drizzled/atomic/sun_studio.h'
--- drizzled/atomic/sun_studio.h	1970-01-01 00:00:00 +0000
+++ drizzled/atomic/sun_studio.h	2009-03-04 02:48:12 +0000
@@ -0,0 +1,138 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_ATOMIC_SUN_STUDIO_H
+#define DRIZZLED_ATOMIC_SUN_STUDIO_H
+
+#define _KERNEL
+#include <atomic.h>
+#undef _KERNEL
+
+inline uint8_t __sync_add_and_fetch(volatile uint8_t* ptr, uint8_t val)
+{
+  (val == 1) ? atomic_inc_8(ptr) : atomic_add_8(ptr, (int8_t)val);
+  return *ptr;
+}
+
+inline uint16_t __sync_add_and_fetch(volatile uint16_t* ptr, uint16_t val)
+{
+  (val == 1) ? atomic_inc_16(ptr) : atomic_add_16(ptr, (int16_t)val);
+  return *ptr;
+}
+
+inline uint32_t __sync_add_and_fetch(volatile uint32_t* ptr, uint32_t val)
+{
+  (val == 1) ? atomic_inc_32(ptr) : atomic_add_32(ptr, (int32_t)val);
+  return *ptr;
+}
+
+# if defined(_KERNEL) || defined(_INT64_TYPE)
+inline uint64_t __sync_add_and_fetch(volatile uint64_t* ptr, uint64_t val)
+{
+  (val == 1) ? atomic_inc_64(ptr) : atomic_add_64(ptr, (int64_t)val);
+  return *ptr;
+}
+# endif /* defined(_KERNEL) || defined(_INT64_TYPE) */
+
+
+inline uint8_t __sync_sub_and_fetch(volatile uint8_t* ptr, uint8_t val)
+{
+  (val == 1) ? atomic_dec_8(ptr) : atomic_add_8(ptr, 0-(int8_t)val);
+  return *ptr;
+}
+
+inline uint16_t __sync_sub_and_fetch(volatile uint16_t* ptr, uint16_t val)
+{
+  (val == 1) ? atomic_dec_16(ptr) : atomic_add_16(ptr, 0-(int16_t)val);
+  return *ptr;
+}
+
+inline uint32_t __sync_sub_and_fetch(volatile uint32_t* ptr, uint32_t val)
+{
+  (val == 1) ? atomic_dec_32(ptr) : atomic_add_32(ptr, 0-(int32_t)val);
+  return *ptr;
+}
+
+# if defined(_KERNEL) || defined(_INT64_TYPE)
+inline uint64_t __sync_sub_and_fetch(volatile uint64_t* ptr, uint64_t val)
+{
+  (val == 1) ? atomic_dec_64(ptr) : atomic_add_64(ptr, 0-(int64_t)val);
+  return *ptr;
+}
+# endif /* defined(_KERNEL) || defined(_INT64_TYPE) */
+
+
+inline uint8_t __sync_lock_test_and_set(volatile uint8_t* ptr, uint8_t val)
+{
+  atomic_swap_8(ptr, val);
+  return *ptr;
+}
+
+inline uint16_t __sync_lock_test_and_set(volatile uint16_t* ptr, uint16_t val)
+{
+  atomic_swap_16(ptr, val);
+  return *ptr;
+}
+
+inline uint32_t __sync_lock_test_and_set(volatile uint32_t* ptr, uint32_t val)
+{
+  atomic_swap_32(ptr, val);
+  return *ptr;
+}
+
+# if defined(_KERNEL) || defined(_INT64_TYPE)
+inline uint64_t __sync_lock_test_and_set(volatile uint64_t* ptr, uint64_t val)
+{
+  atomic_swap_64(ptr, val);
+  return *ptr;
+}
+#endif /* defined(_KERNEL) || defined(_INT64_TYPE) */
+
+inline uint8_t __sync_val_compare_and_swap(volatile uint8_t* ptr,
+                                           uint8_t old_val, uint8_t val)
+{
+  atomic_cas_8(ptr, old_val, val);
+  return *ptr;
+}
+
+inline uint16_t __sync_val_compare_and_swap(volatile uint16_t* ptr,
+                                            uint16_t old_val, uint16_t val)
+{
+  atomic_cas_16(ptr, old_val, val);
+  return *ptr;
+}
+
+inline uint32_t __sync_val_compare_and_swap(volatile uint32_t* ptr,
+                                            uint32_t old_val, uint32_t val)
+{
+  atomic_cas_32(ptr, old_val, val);
+  return *ptr;
+}
+
+# if defined(_KERNEL) || defined(_INT64_TYPE)
+inline uint64_t __sync_val_compare_and_swap(volatile uint64_t* ptr,
+                                            uint64_t old_val, uint64_t val)
+{
+  atomic_cas_64(ptr, old_val, val);
+  return *ptr;
+}
+#endif /* defined(_KERNEL) || defined(_INT64_TYPE) */
+
+
+#endif /* DRIZZLED_ATOMIC_SOLARIS_H */

=== added file 'drizzled/atomics.h'
--- drizzled/atomics.h	1970-01-01 00:00:00 +0000
+++ drizzled/atomics.h	2009-03-04 02:48:12 +0000
@@ -0,0 +1,162 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *  Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_ATOMICS_H
+#define DRIZZLED_ATOMICS_H
+
+#if defined(HAVE_LIBTBB)
+# include <tbb/atomic.h>
+#else
+
+# if defined(__SUNPRO_CC)
+#  include <drizzled/atomic/sun_studio.h>
+# endif
+# if defined(HAVE_GCC_ATOMIC_BUILTINS) || defined(__SUNPRO_CC)
+#  include <drizzled/atomic/gcc_traits.h>
+#  define ATOMIC_TRAITS internal::gcc_traits
+# else  /* use pthread impl */
+#  include <pthread.h>
+#  include <drizzled/atomic/pthread_traits.h>
+#  define ATOMIC_TRAITS internal::pthread_traits
+# endif
+
+
+namespace tbb {
+
+namespace internal {
+
+
+template<typename I>            // Primary template
+struct atomic_base {
+  volatile I my_value;
+};
+
+template<typename I, typename D, typename T >
+class atomic_impl: private atomic_base<I>
+{
+  T traits;
+public:
+  typedef I value_type;
+
+
+  value_type fetch_and_add( D addend )
+  {
+    return traits.fetch_and_add(&this->my_value, addend);
+  }
+
+  value_type fetch_and_increment()
+  {
+    return traits.fetch_and_increment(&this->my_value);
+  }
+
+  value_type fetch_and_decrement()
+  {
+    return traits.fetch_and_decrement(&this->my_value);
+  }
+
+  value_type fetch_and_store( value_type value )
+  {
+    return traits.fetch_and_store(&this->my_value, value);
+  }
+
+  value_type compare_and_swap( value_type value, value_type comparand )
+  {
+    return traits.compare_and_swap(&this->my_value, value, comparand);
+  }
+
+  operator value_type() const volatile
+  {
+    return traits.fetch(&this->my_value);
+  }
+
+  value_type& _internal_reference() const
+  {
+    return this->my_value;
+  }
+
+protected:
+  value_type store_with_release( value_type rhs )
+  {
+    return traits.store_with_release(&this->my_value, rhs);
+  }
+
+public:
+  value_type operator+=( D addend )
+  {
+      return fetch_and_add(addend)+addend;
+  }
+
+  value_type operator-=( D addend )
+  {
+    // Additive inverse of addend computed using binary minus,
+    // instead of unary minus, for sake of avoiding compiler warnings.
+    return operator+=(D(0)-addend);
+  }
+
+  value_type operator++() {
+    return fetch_and_add(1)+1;
+  }
+
+  value_type operator--() {
+    return fetch_and_add(D(-1))-1;
+  }
+
+  value_type operator++(int) {
+    return fetch_and_add(1);
+  }
+
+  value_type operator--(int) {
+    return fetch_and_add(D(-1));
+  }
+
+};
+
+} /* namespace internal */
+
+//! Primary template for atomic.
+/** See the Reference for details.
+    @ingroup synchronization */
+template<typename T>
+struct atomic {
+};
+
+#define __TBB_DECL_ATOMIC(T)                                            \
+  template<> struct atomic<T>                                           \
+  : internal::atomic_impl<T,T,ATOMIC_TRAITS<T,T> > {                    \
+    atomic<T>() : internal::atomic_impl<T,T,ATOMIC_TRAITS<T,T> >() {}   \
+    T operator=( T rhs ) { return store_with_release(rhs); }            \
+  };
+
+__TBB_DECL_ATOMIC(long long)
+__TBB_DECL_ATOMIC(unsigned long long)
+__TBB_DECL_ATOMIC(long)
+__TBB_DECL_ATOMIC(unsigned long)
+__TBB_DECL_ATOMIC(unsigned int)
+__TBB_DECL_ATOMIC(int)
+__TBB_DECL_ATOMIC(unsigned short)
+__TBB_DECL_ATOMIC(short)
+__TBB_DECL_ATOMIC(char)
+__TBB_DECL_ATOMIC(signed char)
+__TBB_DECL_ATOMIC(unsigned char)
+
+}
+#endif /* defined(HAVE_LIBTBB) */
+
+#endif /* DRIZZLED_ATOMIC_H */

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-03-03 08:32:10 +0000
+++ drizzled/drizzled.cc	2009-03-04 02:48:12 +0000
@@ -44,6 +44,9 @@
 #include <drizzled/errmsg.h>
 #include <drizzled/unireg.h>
 #include <drizzled/plugin_scheduling.h>
+
+#include <drizzled/atomics.h>
+
 #include "drizzled/temporal_format.h" /* For init_temporal_formats() */
 
 #if TIME_WITH_SYS_TIME
@@ -129,7 +132,6 @@
 #include <sys/fpu.h>
 #endif
 
-
 inline void setup_fpu()
 {
 #if defined(__FreeBSD__) && defined(HAVE_IEEEFP_H)
@@ -171,8 +173,10 @@
 
 extern "C" void handle_segfault(int sig);
 
+using namespace tbb;
 using namespace std;
 
+
 /* Constants */
 
 const char *show_comp_option_name[]= {"YES", "NO", "DISABLED"};
@@ -291,7 +295,8 @@
 uint32_t delay_key_write_options, protocol_version= PROTOCOL_VERSION;
 uint32_t lower_case_table_names= 1;
 uint32_t tc_heuristic_recover= 0;
-uint32_t volatile thread_count, thread_running;
+uint32_t volatile thread_running;
+atomic<uint32_t> thread_count;
 uint64_t session_startup_options;
 uint32_t back_log;
 uint32_t connect_timeout;
@@ -305,7 +310,7 @@
 uint64_t aborted_threads;
 uint64_t aborted_connects;
 uint64_t max_connect_errors;
-ulong thread_id=1L;
+atomic<my_thread_id> thread_id;
 pid_t current_pid;
 uint64_t slow_launch_threads= 0;
 uint64_t expire_logs_days= 0;
@@ -444,10 +449,9 @@
 extern scheduling_st thread_scheduler;
 
 /**
-  Number of currently active user connections. The variable is protected by
-  LOCK_thread_count.
-*/
-uint32_t connection_count= 0;
+ * Number of currently active user connections.
+ */
+atomic<uint32_t> connection_count;
 
 /* Function declarations */
 
@@ -1000,11 +1004,10 @@
 {
   session->cleanup();
 
-  (void) pthread_mutex_lock(&LOCK_thread_count);
   connection_count--;
   thread_count--;
   delete session;
-  pthread_mutex_unlock(&LOCK_thread_count);
+
   return;
 }
 
@@ -1080,8 +1083,8 @@
   fprintf(stderr, "read_buffer_size=%ld\n", (long) global_system_variables.read_buff_size);
   fprintf(stderr, "max_used_connections=%u\n", max_used_connections);
   fprintf(stderr, "max_threads=%u\n", thread_scheduler.max_threads);
-  fprintf(stderr, "thread_count=%u\n", thread_count);
-  fprintf(stderr, "connection_count=%u\n", connection_count);
+  fprintf(stderr, "thread_count=%u\n", uint32_t(thread_count));
+  fprintf(stderr, "connection_count=%u\n", uint32_t(connection_count));
   fprintf(stderr, _("It is possible that drizzled could use up to \n"
                     "key_buffer_size + (read_buffer_size + "
                     "sort_buffer_size)*max_threads = %"PRIu64" K\n"
@@ -1896,10 +1899,8 @@
 
 static void create_new_thread(Session *session)
 {
-  pthread_mutex_lock(&LOCK_thread_count);
-
-  ++connection_count;
-
+
+  connection_count++;
   if (connection_count > max_used_connections)
     max_used_connections= connection_count;
 
@@ -1908,13 +1909,14 @@
     the embedded library.
     TODO: refactor this to avoid code duplication there
   */
-  session->thread_id= session->variables.pseudo_thread_id= thread_id++;
+  session->thread_id= session->variables.pseudo_thread_id= ++thread_id;
 
   thread_count++;
 
   /* 
     If we error on creation we drop the connection and delete the session.
   */
+  pthread_mutex_lock(&LOCK_thread_count);
   if (thread_scheduler.add_connection(session))
   {
     char error_message_buff[DRIZZLE_ERRMSG_SIZE];
@@ -1926,11 +1928,9 @@
     /* Can't use my_error() since store_globals has not been called. */
     snprintf(error_message_buff, sizeof(error_message_buff), ER(ER_CANT_CREATE_THREAD), 1); /* TODO replace will better error message */
     net_send_error(session, ER_CANT_CREATE_THREAD, error_message_buff);
-    (void) pthread_mutex_lock(&LOCK_thread_count);
-    --connection_count;
-    session->close_connection(0, 0);
+    connection_count--;
+    session->close_connection(0, true);
     delete session;
-    (void) pthread_mutex_unlock(&LOCK_thread_count);
   }
 }
 
@@ -2925,6 +2925,9 @@
   if (!(tmpenv = getenv("MY_BASEDIR_VERSION")))
     tmpenv = PREFIX;
   (void) strncpy(drizzle_home, tmpenv, sizeof(drizzle_home)-1);
+
+  connection_count= 0;
+  thread_id= 1;
 }
 
 

=== modified file 'drizzled/query_id.cc'
--- drizzled/query_id.cc	2009-02-11 20:09:54 +0000
+++ drizzled/query_id.cc	2009-03-04 02:48:12 +0000
@@ -21,16 +21,15 @@
 #include <drizzled/definitions.h>
 #include <drizzled/query_id.h>
 #include <mysys/my_pthread.h>
+#include <drizzled/atomics.h>
 
-Query_id::Query_id() : the_query_id(1)
+Query_id::Query_id()
 {
-  /* pthread_mutex_init always returns 0 */
-  (void)pthread_mutex_init(&LOCK_query_id, MY_MUTEX_INIT_FAST);
+  the_query_id= 1;
 }
 
 Query_id::~Query_id()
 {
-  pthread_mutex_destroy(&LOCK_query_id);
 }
 
 query_id_t Query_id::value() const
@@ -40,10 +39,6 @@
 
 query_id_t Query_id::next()
 {
-  pthread_mutex_lock(&LOCK_query_id);
-  query_id_t ret= the_query_id++;
-  pthread_mutex_unlock(&LOCK_query_id);
-
-  return ret;
+  return ++the_query_id;
 }
 

=== modified file 'drizzled/query_id.h'
--- drizzled/query_id.h	2008-10-28 15:39:49 +0000
+++ drizzled/query_id.h	2009-03-04 02:48:12 +0000
@@ -20,6 +20,8 @@
 #ifndef DRIZZLED_GLOBAL_QUERY_ID_H
 #define DRIZZLED_GLOBAL_QUERY_ID_H
 
+#include <drizzled/atomics.h>
+
 #include <pthread.h>
 
 class Query_id
@@ -38,8 +40,7 @@
   query_id_t next();
 
 private:
-  pthread_mutex_t LOCK_query_id;
-  query_id_t the_query_id;
+  tbb::atomic<query_id_t> the_query_id;
 
   Query_id();
   Query_id(Query_id const&);

=== modified file 'drizzled/server_includes.h'
--- drizzled/server_includes.h	2009-02-28 17:49:22 +0000
+++ drizzled/server_includes.h	2009-03-04 02:48:12 +0000
@@ -115,7 +115,6 @@
 extern uint64_t log_10_int[20];
 extern uint64_t keybuff_size;
 extern uint64_t session_startup_options;
-extern ulong thread_id;
 extern uint64_t aborted_threads;
 extern uint64_t aborted_connects;
 extern uint64_t slow_launch_threads;
@@ -144,8 +143,7 @@
 extern ulong log_output_options;
 extern bool opt_character_set_client_handshake;
 extern bool volatile abort_loop, shutdown_in_progress;
-extern uint32_t volatile thread_count, thread_running, global_read_lock;
-extern uint32_t connection_count;
+extern uint32_t volatile thread_running, global_read_lock;
 extern bool opt_sql_bin_update;
 extern bool opt_safe_user_create;
 extern bool opt_no_mix_types;
@@ -167,7 +165,7 @@
        LOCK_timezone,
        LOCK_global_read_lock,
        LOCK_global_system_variables, LOCK_user_conn,
-       LOCK_bytes_sent, LOCK_bytes_received, LOCK_connection_count;
+       LOCK_bytes_sent, LOCK_bytes_received;
 extern pthread_mutex_t LOCK_server_started;
 extern pthread_rwlock_t LOCK_sys_init_connect;
 extern pthread_rwlock_t LOCK_system_variables_hash;

